
#include <Arduino.h>
#include <Wire.h>
#include "Keyboard.h"
#include "Rfid.h"
#include "Interface.h"
#include <ESP8266WiFi.h>
#include <vector>
#include "Users.h"
// Rfid
Rfid rfid;
// i2C Bus
#define PIN_WIRE_SDA D3
#define PIN_WIRE_SCL D4

Keyboard keyboard(200);
Interface iface;
// Wifi control
IPAddress local_IP(192,168,4,22);
IPAddress gateway(192,168,4,9);
IPAddress subnet(255,255,255,0);

// User DB

typedef struct
{
	String uid;
	String first_name;
	String last_name;
	String rfid_uid;
	String user_pin;
} user_account;
std::vector<user_account> userdb;
void setup()
{
	Serial.begin(9600);
	Serial.print("Starting");
	Serial.println(WiFi.softAPConfig(local_IP, gateway, subnet) ? "Ready" : "Failed!");
	Serial.println(WiFi.softAP("Doorlock") ? "Ready" : "Failed!");
	//delay(500);
	keyboard.begin(&Wire);
	rfid.begin();
	iface.begin(&keyboard);
	userdb.push_back(user_account{.uid = "010202", .first_name = "Max", .last_name = "Mustermann and Sons", .rfid_uid = "D3 4E 03 0B", .user_pin = "1212"});
	userdb.push_back(user_account{.uid = "012345", .first_name = "John", .last_name = "Doe", .rfid_uid = "8C 75 6C 17", .user_pin = "2424"});
}

void loop()
{
	keyboard.scanAsync();
	rfid.scan();
	std::vector<user_account> logon_users;
	if (iface.pinAvailable())
	{
		String entered_pin = iface.getPin();
		std::copy_if(userdb.begin(), userdb.end(), std::back_inserter(logon_users), [=](user_account user)
					 { return entered_pin == user.user_pin; });
		if (logon_users.size() == 0)
			iface.showMessage("Login failed!", "-> Pin incorrect",3000);
	}
	else if (rfid.available())
	{
		String rfid_card_uid = rfid.getID();
		std::copy_if(userdb.begin(), userdb.end(), std::back_inserter(logon_users), [=](user_account user)
					 { return rfid_card_uid == user.rfid_uid; });
		if (logon_users.size() == 0)
			iface.showMessage("Login failed!", "-> Unkown Card",3000);
	}
	if (logon_users.size() > 0)
	{
		iface.greetUser(logon_users[0].first_name + " " + logon_users[0].last_name);
		logon_users.clear();
	}
	iface.render();
}